/* This file is in the public domain */

#include <string>
#include <cstdlib>
#include <ctime>

#include <botan/filters.h>

#include <botan/randpool.h>
#include <botan/x917_rng.h>
#include <botan/rng.h>

using namespace Botan;

/* Not too useful generally; just dumps random bits for benchmarking */
class RNG_Filter : public Filter
   {
   public:
      void write(const byte[], u32bit);
      RNG_Filter(RandomNumberGenerator* r) : rng(r), buffer(1024)
         {
         Global_RNG::randomize(buffer, buffer.size());
         rng->add_entropy(buffer, buffer.size());
         }
      ~RNG_Filter() { delete rng; }
   private:
      RandomNumberGenerator* rng;
      SecureVector<byte> buffer;
   };

void RNG_Filter::write(const byte[], u32bit length)
   {
   while(length)
      {
      u32bit gen = std::min(buffer.size(), length);
      rng->randomize(buffer, gen);
      length -= gen;
      }
   }

Filter* lookup_rng(const std::string& algname)
   {
   if(algname == "X9.17")
      return new RNG_Filter(new ANSI_X917_RNG);
   if(algname == "Randpool")
      return new RNG_Filter(new Randpool);
   return 0;
   }
