/*
Prints the message digest of files, using an arbitrary hash function
chosen by the user. This is less flexible that I might like, for example:
   ./hash sha1 some_file [or md5 or sha-1 or ripemd160 or ...]
will not work, cause the name lookup is case-sensitive. Oh well...

Written by Jack Lloyd (lloyd@randombit.net), on August 4, 2002

This file is in the public domain
*/

#include <iostream>
#include <fstream>
#include <botan/filters.h>
#include <botan/lookup.h>

int main(int argc, char* argv[])
   {
   if(argc < 3)
      {
      std::cout << "Usage: hash digest <filenames>" << std::endl;
      return 1;
      }

   Botan::LibraryInitializer init;

   try {

   if(!Botan::have_hash(argv[1]))
      {
      std::cout << "Unknown hash \"" << argv[1] << "\"" << std::endl;
      return 1;
      }

   Botan::Pipe pipe(new Botan::Hash_Filter(argv[1]),
                    new Botan::Hex_Encoder);

   int skipped = 0;
   for(int j = 2; argv[j] != 0; j++)
      {
      std::ifstream file(argv[j]);
      if(!file)
         {
         std::cout << "ERROR: could not open " << argv[j] << std::endl;
         skipped++;
         continue;
         }
      pipe.start_msg();
      file >> pipe;
      pipe.end_msg();
      pipe.set_default_msg(j-2-skipped);
      std::cout << pipe << "  " << argv[j] << std::endl;
      }
   }
   catch(std::exception& e)
      {
      std::cout << "Exception caught: " << e.what() << std::endl;
      }
   return 0;
   }
