/*
Generate a 1024 bit RSA key, and then create a PKCS #10 certificate request for
that key. The private key will be encrypted with PKCS #8 and stored in another
file.

Written by Jack Lloyd (lloyd@randombit.net), April 7, 2003

This file is in the public domain
*/
#include <botan/x509self.h>
#include <botan/rsa.h>
#include <botan/conf.h>
using namespace Botan;

#include <iostream>
#include <fstream>

int main(int argc, char* argv[])
   {
   if(argc != 6)
      {
      std::cout << "Usage: " << argv[0] <<
         " passphrase name country_code organization email" << std::endl;
      return 1;
      }

   LibraryInitializer init;

   try {

   RSA_PrivateKey rsa_key(1024);

   std::ofstream priv_key("private.pem");
   priv_key << PKCS8::PEM_encode(rsa_key, argv[1]);

   X509_Cert_Options opts;

   opts.common_name = argv[2];
   opts.country = argv[3];
   opts.organization = argv[4];
   opts.email = argv[5];

   /* Some hard-coded options, just to give you an idea of what's available */
   opts.challenge = "a fixed challenge passphrase";
   opts.locality = "Baltimore";
   opts.state = "MD";
   opts.org_unit = "Testing";
   opts.add_ex_constraint("PKIX.ClientAuth");
   opts.add_ex_constraint("PKIX.IPsecUser");
   opts.add_ex_constraint("PKIX.EmailProtection");

   PKCS10_Request req = X509::create_cert_req(opts, rsa_key);

   std::ofstream req_file("req.pem");
   req_file << req.PEM_encode();

   }
   catch(std::exception& e)
      {
      std::cout << "Exception: " << e.what() << std::endl;
      return 1;
      }

   return 0;
   }
