/*************************************************
* BigInt Multiplication Algorithms Header File   *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#ifndef BOTAN_MP_MUL_H__
#define BOTAN_MP_MUL_H__

#include <botan/mp_types.h>

namespace Botan {

/*************************************************
* MP Multiplication Algorithms                   *
*************************************************/
bool try_fast_multiply(word* const, u32bit,
                       const word*, u32bit, u32bit,
                       const word*, u32bit, u32bit);

/*************************************************
* Karatsuba Multiplication                       *
*************************************************/
void bigint_karat16(word[32], const word[16], const word[16]);
void bigint_karat32(word[64], const word[32], const word[32]);
void bigint_karat64(word[128], const word[64], const word[64]);
void bigint_karat128(word[256], const word[128], const word[128]);

}

extern "C" {

/*************************************************
* Comba Multiplication                           *
*************************************************/
void bigint_comba4(word[8], const word[4], const word[4]);
void bigint_comba8(word[16], const word[8], const word[8]);

/*************************************************
* Simple O(N^2) Multiplication                   *
*************************************************/
void bigint_simple_mul(word* const, const word*, u32bit,
                                    const word*, u32bit);

}

#endif
