/*************************************************
* Unix EntropySource Header File                 *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/buf_es.h>
#include <vector>

#ifndef BOTAN_EXT_ENTROPY_SRC_UNIX_H__
#define BOTAN_EXT_ENTROPY_SRC_UNIX_H__

namespace Botan {

/*************************************************
* Unix Program Entry                             *
*************************************************/
struct Unix_Program
   {
   Unix_Program(const std::string& n, const std::string a, u32bit p)
      { name = n; args = a; priority = p; working = true; }
   Unix_Program(const std::string& n, u32bit p)
      { name = n; priority = p; working = true; }
   std::string name, args;
   u32bit priority;
   bool working;
   };

/*************************************************
* Unix Entropy Source                            *
*************************************************/
class Unix_EntropySource : public Buffered_EntropySource
   {
   public:
      void add_sources(const Unix_Program[], u32bit);

      Unix_EntropySource(bool = true);
      Unix_EntropySource(const Unix_Program[], u32bit, bool = true);
   private:
      static const Unix_Program DEFAULT_SOURCES[];
      void do_fast_poll();
      void do_slow_poll();
      void add_default_sources();
      void gather(u32bit, u32bit);
      u32bit gather_entropy(Unix_Program&);
      void timestamp();
      std::vector<Unix_Program> sources;
   };

}

#endif
